<?php

/**
 * The REST API functionality of the plugin.
 *
 * @link       https://bigblue.co
 * @since      1.0.0
 *
 * @package    Bigblue
 * @subpackage Bigblue/rest
 */

/**
 * The REST API functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Bigblue
 * @subpackage Bigblue/rest
 * @author     Bigblue Integrations <tech@bigblue.io>
 */
class Bigblue_Rest {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
        $this->version = $version;

        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'store/class-bigblue-config-store.php';
        
	}

    public function register_routes() {
        register_rest_route(
            'bigblue/v1', '/version', array(
                'methods' => 'GET',
                'callback' => array( $this, 'handle_get_version' ),
                'permission_callback' => '__return_true'
            )
        );

        register_rest_route(
            'bigblue/v1', '/storefront-access-token', array(
                'methods' => 'POST',
                'callback' => array( $this, 'handle_post_sat' ),
                'permission_callback' => array( $this, 'authenticate' )
            ) 
        );
    }

    public function handle_get_version() {
        $this->send_api_response( 200, array(
            "version" => $this->version
        ) );
    }

    public function handle_post_sat( $request ) {

        $body=$request->get_body();

        if ( $body === null ) {
            $this->send_api_response( 400 );
        }

        $parsed = (array) json_decode($body);

        if ( !isset($parsed['token']) || !isset($parsed['expiry']) ) {
            $this->send_api_response( 400 );
        }

        Bigblue_Config_Store::set_storefront_access_token( $parsed['token'], $parsed['expiry'] );

        $this->send_api_response( 200 );
    }

    public function authenticate( $request ) {

        $public_key = self::read_public_key();
        $body = $request->get_body();

        switch (openssl_verify($body, base64_decode( $request['sig'] ), $public_key, OPENSSL_ALGO_SHA256)) {
            case -1:
                // echo openssl_error_string();
                return false;
            case 1:
                return true;
            default:
                return false;
        }
    }

    /**
     * Send API request response.
     *
     * @param int $code http code
     * @param mixed $body to send along response
     *
     * @void
     */
    protected function send_api_response( $code, $body = null ) {
        header( 'X-Bigblue-Module-Version: ' . $this->version );
        http_response_code( $code );
        header( 'Content-Type: application/json; charset=utf-8' );
        if ( $body !== null ) {
            echo json_encode( $body );
        }
        die();
    }
   
    protected static function read_public_key() {
        $path = realpath(dirname(dirname(__FILE__))) . DIRECTORY_SEPARATOR
        . 'assets' . DIRECTORY_SEPARATOR . 'pub.pem';

        return openssl_get_publickey("file://".$path);
    }
}
